@@ -35,14 +35,18 @@ OC.L10N.register(
     "Send" : "שליחה",
     "Choose a file to add as attachment" : "בחירת קובץ להוספה כקובץ מצורף",
     "No messages in this folder" : "אין הודעות בתיקייה זו",
+    "Favorite" : "סימון כמועדף",
     "Mark read" : "סימון כנקרא",
     "Delete" : "מחיקה",
+    "to" : "אל",
+    "cc" : "עותק",
     "Reply" : "תגובה",
     "Forward" : "העברה",
     "Could not load your message" : "לא ניתן לטעון את ההודעה שלך",
     "Import into calendar" : "ייבוא ליומן",
     "Download attachment" : "הורדת קובץ מצורף",
     "Save to Files" : "שמירה לקבצים",
+    "Unnamed" : "ללא שם",
     "Choose a folder to store the attachment in" : "יש לבחור תיקייה לשמירת הקובץ המצורף",
     "Save all to Files" : "שמירת הכו לקבצים",
     "The images have been blocked to protect your privacy." : "התמונות נחסמו כדי להגן על הפרטיות שלך",
@@ -67,6 +71,15 @@ OC.L10N.register(
     "Search" : "חיפוש",
     "Refresh" : "רענון",
     "Connect your mail account" : "התחברות לחשבון הדוא״ל שלך",
+    "All" : "הכול",
+    "Archive" : "לארכיון",
+    "Drafts" : "טיוטות",
+    "Favorites" : "מועדפים",
+    "All inboxes" : "כל תיבות דואר נכנס",
+    "Inbox" : "דואר נכנס",
+    "Junk" : "דואר זבל",
+    "Sent" : "נשלח",
+    "Trash" : "פח אשפה",
     "{from}\n{subject}" : "{from}\n{subject}",
     "_%n new message \nfrom {from}_::_%n new messages \nfrom {from}_" : ["הודעה חדשה \nמאת {from}","2 הודעות חדשות \nמאת {from}","%n הודעות חדשות \nמאת {from}","%n הודעות חדשות \nמאת {from}"],
     "Nextcloud Mail" : "דוא״ל Nextcloud",
