@@ -11,6 +11,7 @@ OC.L10N.register(
     "Sending …" : "កំពុង​ផ្ញើរ...",
     "Send" : "ផ្ញើ",
     "Delete" : "លុប",
+    "to" : "ទៅ",
     "Reply" : "ឆ្លើយតប",
     "Settings" : "ការកំណត់",
     "Add folder" : "Add folder",
@@ -19,6 +20,13 @@ OC.L10N.register(
     "User" : "User",
     "Host" : "ម៉ាស៊ីន​ផ្ទុក",
     "Port" : "ច្រក",
-    "Search" : "Search"
+    "Search" : "Search",
+    "All" : "ទាំងអស់",
+    "Drafts" : "ពង្រាង",
+    "Favorites" : "Favorites",
+    "Inbox" : "ក្នុង​ប្រអប់សារ",
+    "Junk" : "ព្រៃ",
+    "Sent" : "ផ្ញើ",
+    "Trash" : "ធុង​សំរាម"
 },
 "nplurals=1; plural=0;");
