@@ -21,8 +21,11 @@ OC.L10N.register(
     "Retry" : "Reîncearcă",
     "Message sent!" : "Mesaj trimis!",
     "Send" : "Trimite",
+    "Favorite" : "Favorite",
     "Delete" : "Șterge",
+    "to" : "către",
     "Reply" : "Răspunde",
+    "Unnamed" : "Fără nume",
     "Choose a folder to store the attachment in" : "Alege un dosar în care să stochez atașamentul",
     "Settings" : "Setări",
     "Add folder" : "Add folder",
@@ -37,6 +40,12 @@ OC.L10N.register(
     "Port" : "Portul",
     "Search" : "Caută",
     "Refresh" : "Reîmprospătează",
+    "All" : "Toate ",
+    "Archive" : "Arhivă",
+    "Drafts" : "Schițe",
+    "Favorites" : "Favorite",
+    "Sent" : "Trimis",
+    "Trash" : "Gunoi",
     "{from}\n{subject}" : "{from}\n{subject}",
     "Error loading message" : "Eroare la încărcarea mesajelor",
     "Redirect" : "Deviere",
