@@ -37,7 +37,10 @@ use OCA\Mail\Exception\ServiceException;
 use OCA\Mail\Model\IMAPMessage;
 use OCP\AppFramework\Db\DoesNotExistException;
 use OCP\ILogger;
+use function array_filter;
+use function array_map;
 use function iterator_to_array;
+use function reset;
 
 class MessageMapper {
 
@@ -66,6 +69,109 @@ class MessageMapper {
 		return $result[0];
 	}
 
+	/**
+	 * @param Horde_Imap_Client_Socket $client
+	 * @param string $mailbox
+	 *
+	 * @param int $maxResults
+	 * @param int $highestKnownUid
+	 *
+	 * @return array
+	 * @throws Horde_Imap_Client_Exception
+	 */
+	public function findAll(Horde_Imap_Client_Socket $client,
+							string $mailbox,
+							int $maxResults,
+							int $highestKnownUid): array {
+		/**
+		 * To prevent memory exhaustion, we don't want to just ask for a list of
+		 * all UIDs and limit them client-side. Instead we can (hopefully
+		 * efficiently) query the min and max UID as well as the number of
+		 * messages. Based on that we assume that UIDs are somewhat distributed
+		 * equally and build a page to fetch.
+		 *
+		 * This logic might return fewer or more results than $maxResults
+		 */
+
+		$metaResults = $client->search(
+			$mailbox,
+			null,
+			[
+				'results' => [
+					Horde_Imap_Client::SEARCH_RESULTS_MIN,
+					Horde_Imap_Client::SEARCH_RESULTS_MAX,
+					Horde_Imap_Client::SEARCH_RESULTS_COUNT,
+				]
+			]
+		);
+		/** @var int $min */
+		$min = $metaResults['min'];
+		/** @var int $max */
+		$max = $metaResults['max'];
+		/** @var int $total */
+		$total = $metaResults['count'];
+
+		if ($total === 0) {
+			// Nothing to fetch for this mailbox
+			return [
+				'messages' => [],
+				'all' => true,
+			];
+		}
+
+		// The inclusive range of UIDs
+		$totalRange = $max - $min + 1;
+		// Here we assume somewhat equally distributed UIDs
+		// +1 is added to fetch all messages with the rare case of strictly
+		// continuous UIDs and fractions
+		$estimatedPageSize = (int)(($totalRange / $total) * $maxResults) + 1;
+		// Determine min UID to fetch, but don't exceed the known maximum
+		$lower = max(
+			$min,
+			($highestKnownUid ?? 0) + 1
+		);
+		// Determine max UID to fetch, but don't exceed the known maximum
+		$upper = min(
+			$max,
+			$lower + $estimatedPageSize
+		);
+		$this->logger->debug("Built range for findAll: min=$min max=$max total=$total totalRange=$totalRange estimatedPageSize=$estimatedPageSize lower=$lower upper=$upper highestKnownUid=$highestKnownUid");
+
+		$query = new Horde_Imap_Client_Fetch_Query();
+		$query->uid();
+		$uidsToFetch = array_slice(
+			array_filter(
+				array_map(
+					function (Horde_Imap_Client_Data_Fetch $data) {
+						return $data->getUid();
+					},
+					iterator_to_array($client->fetch(
+						$mailbox,
+						$query,
+						[
+							'ids' => new Horde_Imap_Client_Ids($lower . ':' . $upper)
+						]
+					))
+				),
+
+				function (int $uid) use ($highestKnownUid) {
+					// Don't load the ones we already know
+					return $highestKnownUid === null || $uid > $highestKnownUid;
+				}
+			),
+			0,
+			$maxResults
+		);
+		return [
+			'messages' => $this->findByIds(
+				$client,
+				$mailbox,
+				$uidsToFetch
+			),
+			'all' => $upper === $max,
+		];
+	}
+
 	/**
 	 * @return IMAPMessage[]
 	 * @throws Horde_Imap_Client_Exception
@@ -77,10 +183,8 @@ class MessageMapper {
 		$query = new Horde_Imap_Client_Fetch_Query();
 		$query->envelope();
 		$query->flags();
-		$query->size();
 		$query->uid();
 		$query->imapDate();
-		$query->structure();
 
 		$fetchResults = iterator_to_array($client->fetch($mailbox, $query, [
 			'ids' => new Horde_Imap_Client_Ids($ids),
@@ -137,7 +241,7 @@ class MessageMapper {
 	}
 
 	public function markAllRead(Horde_Imap_Client_Base $client,
-								string $mailbox) {
+								string $mailbox): void {
 		$client->store($mailbox, [
 			'add' => [
 				Horde_Imap_Client::FLAG_SEEN,
@@ -216,6 +320,58 @@ class MessageMapper {
 		);
 	}
 
+	/**
+	 * @throws Horde_Imap_Client_Exception
+	 */
+	public function removeFlag(Horde_Imap_Client_Socket $client,
+							   Mailbox $mailbox,
+							   int $uid,
+							   string $flag): void {
+		$client->store(
+			$mailbox->getName(),
+			[
+				'ids' => new Horde_Imap_Client_Ids($uid),
+				'remove' => [$flag],
+			]
+		);
+	}
+
+	/**
+	 * @param Horde_Imap_Client_Socket $client
+	 * @param string $mailbox
+	 * @param int $id
+	 *
+	 * @return string|null
+	 * @throws ServiceException
+	 */
+	public function getSource(Horde_Imap_Client_Socket $client,
+							  string $mailbox,
+							  int $id): ?string {
+		$query = new Horde_Imap_Client_Fetch_Query();
+		$query->uid();
+		$query->fullText([
+			'peek' => true,
+		]);
+
+		try {
+			$result = iterator_to_array($client->fetch($mailbox, $query, [
+				'ids' => new Horde_Imap_Client_Ids($id),
+			]), false);
+		} catch (Horde_Imap_Client_Exception $e) {
+			throw new ServiceException("Could not fetch message source: " . $e->getMessage(), $e->getCode(), $e);
+		}
+
+		$msg = array_map(function (Horde_Imap_Client_Data_Fetch $result) {
+			return $result->getFullMsg();
+		}, $result);
+
+		if (empty($msg)) {
+			return null;
+		}
+
+		return reset($msg);
+	}
+
 	public function getHtmlBody(Horde_Imap_Client_Socket $client,
 								string $mailbox,
 								int $id): ?string {
@@ -251,7 +407,6 @@ class MessageMapper {
 					'peek' => true
 				]);
 			}
-
 		}
 
 		$parts = $client->fetch($mailbox, $partsQuery, [
@@ -260,23 +415,14 @@ class MessageMapper {
 
 		foreach ($parts as $part) {
 			/** @var Horde_Imap_Client_Data_Fetch $part */
-			$stream = $part->getBodyPart($htmlPartId, true);
-			$partData = $structure->getPart($htmlPartId);
-			$partData->setContents($stream, [
-				'usestream' => true,
-			]);
-
 			$body = $part->getBodyPart($htmlPartId);
 			if ($body !== null) {
-				$structurePart = $structure[$htmlPartId];
 				$mimeHeaders = $part->getMimeHeader($htmlPartId, Horde_Imap_Client_Data_Fetch::HEADER_PARSE);
 				if ($enc = $mimeHeaders->getValue('content-transfer-encoding')) {
 					$structure->setTransferEncoding($enc);
 				}
 				$structure->setContents($body);
-				$decoded = $structure->getContents();
-
-				return $decoded;
+				return $structure->getContents();
 			}
 		}
 
@@ -302,7 +448,7 @@ class MessageMapper {
 		$partsQuery->fullText();
 		foreach ($structure->partIterator() as $part) {
 			/** @var Horde_Mime_Part $part */
-			if ($part->getMimeId() === "0") {
+			if ($part->getMimeId() === '0') {
 				// Ignore message header
 				continue;
 			}
@@ -346,4 +492,68 @@ class MessageMapper {
 		return $attachments;
 	}
 
+	/**
+	 * @param Horde_Imap_Client_Socket $client
+	 * @param int[] $uids
+	 *
+	 * @return MessageStructureData[]
+	 * @throws Horde_Imap_Client_Exception
+	 */
+	public function getBodyStructureData(Horde_Imap_Client_Socket $client,
+										 string $mailbox,
+										 array $uids): array {
+		$structureQuery = new Horde_Imap_Client_Fetch_Query();
+		$structureQuery->structure();
+
+		$structures = $client->fetch($mailbox, $structureQuery, [
+			'ids' => new Horde_Imap_Client_Ids($uids),
+		]);
+
+		return array_map(function (Horde_Imap_Client_Data_Fetch $fetchData) use ($mailbox, $client) {
+			$hasAttachments = false;
+			$text = '';
+
+			$structure = $fetchData->getStructure();
+			foreach ($structure as $part) {
+				if ($part instanceof Horde_Mime_Part && $part->isAttachment()) {
+					$hasAttachments = true;
+					break;
+				}
+			}
+
+			$textBodyId = $structure->findBody('text');
+			// $htmlBodyId = $structure->findBody('html');
+			// $htmlBody = $data->getBodyPart($htmlBodyId);
+
+			$partsQuery = new Horde_Imap_Client_Fetch_Query();
+			if ($textBodyId === null) {
+				return new MessageStructureData($hasAttachments, $text);
+			}
+			$partsQuery->bodyPart($textBodyId, [
+				'decode' => true,
+				'peek' => true,
+			]);
+			$partsQuery->mimeHeader($textBodyId, [
+				'peek' => true
+			]);
+			$parts = $client->fetch($mailbox, $partsQuery, [
+				'ids' => new Horde_Imap_Client_Ids([$fetchData->getUid()]),
+			]);
+			/** @var Horde_Imap_Client_Data_Fetch $part */
+			$part = $parts[$fetchData->getUid()];
+			$body = $part->getBodyPart($textBodyId);
+
+			if (!empty($body)) {
+				$mimeHeaders = $fetchData->getMimeHeader($textBodyId, Horde_Imap_Client_Data_Fetch::HEADER_PARSE);
+				if ($enc = $mimeHeaders->getValue('content-transfer-encoding')) {
+					$structure->setTransferEncoding($enc);
+				}
+				$structure->setContents($body);
+				/** @var string $text */
+				$text = $structure->getContents();
+			}
+
+			return new MessageStructureData($hasAttachments, $text);
+		}, iterator_to_array($structures->getIterator()));
+	}
 }
