@@ -23,63 +23,55 @@ declare(strict_types=1);
 
 namespace OCA\Mail\IMAP\Sync;
 
+use Horde_Imap_Client;
 use Horde_Imap_Client_Base;
 use Horde_Imap_Client_Exception;
 use Horde_Imap_Client_Exception_Sync;
 use Horde_Imap_Client_Ids;
 use Horde_Imap_Client_Mailbox;
+use OCA\Mail\IMAP\MessageMapper;
+use OCA\mail\lib\Exception\UidValidityChangedException;
 
 class Synchronizer {
 
-	/** @var ISyncStrategy */
-	private $simpleSync;
+	/** @var MessageMapper */
+	private $messageMapper;
 
-	/** @var ISyncStrategy */
-	private $favSync;
-
-	/**
-	 * @param SimpleMailboxSync $simpleSync
-	 * @param FavouritesMailboxSync $favSync
-	 */
-	public function __construct(SimpleMailboxSync $simpleSync,
-								FavouritesMailboxSync $favSync) {
-		$this->simpleSync = $simpleSync;
-		$this->favSync = $favSync;
+	public function __construct(MessageMapper $messageMapper) {
+		$this->messageMapper = $messageMapper;
 	}
 
 	/**
 	 * @param Horde_Imap_Client_Base $imapClient
 	 * @param Request $request
+	 * @param int $criteria
+	 *
 	 * @return Response
 	 * @throws Horde_Imap_Client_Exception
 	 * @throws Horde_Imap_Client_Exception_Sync
+	 * @throws UidValidityChangedException
 	 */
-	public function sync(Horde_Imap_Client_Base $imapClient, Request $request): Response {
+	public function sync(Horde_Imap_Client_Base $imapClient,
+						 Request $request,
+						 int $criteria = Horde_Imap_Client::SYNC_NEWMSGSUIDS|Horde_Imap_Client::SYNC_FLAGSUIDS|Horde_Imap_Client::SYNC_VANISHEDUIDS): Response {
 		$mailbox = new Horde_Imap_Client_Mailbox($request->getMailbox());
 		$ids = new Horde_Imap_Client_Ids($request->getUids());
-		$hordeSync = $imapClient->sync($mailbox, $request->getToken(), [
-			'ids' => $ids
-		]);
-
-		$syncStrategy = $this->getSyncStrategy($request);
-		$newMessages = $syncStrategy->getNewMessages($imapClient, $request, $hordeSync);
-		$changedMessages = $syncStrategy->getChangedMessages($imapClient, $request, $hordeSync);
-		$vanishedMessages = $syncStrategy->getVanishedMessages($imapClient, $request, $hordeSync);
-
-		$newSyncToken = $imapClient->getSyncToken($request->getMailbox());
-		return new Response($newSyncToken, $newMessages, $changedMessages, $vanishedMessages);
-	}
-
-	/**
-	 * @param Request $request
-	 * @return ISyncStrategy
-	 */
-	private function getSyncStrategy(Request $request): ISyncStrategy {
-		if ($request->isFlaggedMailbox()) {
-			return $this->favSync;
-		} else {
-			return $this->simpleSync;
+		try {
+			$hordeSync = $imapClient->sync($mailbox, $request->getToken(), [
+				'criteria' => $criteria,
+				'ids' => $ids
+			]);
+		} catch (Horde_Imap_Client_Exception_Sync $e) {
+			if ($e->getCode() === Horde_Imap_Client_Exception_Sync::UIDVALIDITY_CHANGED) {
+				throw new UidValidityChangedException();
+			}
+			throw $e;
 		}
-	}
 
+		$newMessages = $this->messageMapper->findByIds($imapClient, $request->getMailbox(), $hordeSync->newmsgsuids->ids);
+		$changedMessages = $this->messageMapper->findByIds($imapClient, $request->getMailbox(), $hordeSync->flagsuids->ids);
+		$vanishedMessageUids = $hordeSync->vanisheduids->ids;
+
+		return new Response($newMessages, $changedMessages, $vanishedMessageUids);
+	}
 }
