@@ -23,11 +23,9 @@ declare(strict_types=1);
 
 namespace OCA\Mail\Service;
 
-use Exception;
 use Horde_Exception;
 use Horde_Imap_Client;
 use Horde_Imap_Client_Exception;
-use Horde_Imap_Client_Mailbox;
 use Horde_Mail_Transport_Null;
 use Horde_Mime_Exception;
 use Horde_Mime_Headers_Date;
@@ -107,16 +105,16 @@ class MailTransmission implements IMailTransmission {
 	/**
 	 * Send a new message or reply to an existing one
 	 *
-	 * @param string $userId
 	 * @param NewMessageData $messageData
 	 * @param RepliedMessageData $replyData
 	 * @param Alias|null $alias
 	 * @param int|null $draftUID
 	 *
 	 * @throws ServiceException
+	 *
+	 * @return void
 	 */
-	public function sendMessage(string $userId,
-								NewMessageData $messageData,
+	public function sendMessage(NewMessageData $messageData,
 								RepliedMessageData $replyData = null,
 								Alias $alias = null,
 								int $draftUID = null) {
@@ -137,7 +135,7 @@ class MailTransmission implements IMailTransmission {
 		$message->setCC($messageData->getCc());
 		$message->setBcc($messageData->getBcc());
 		$message->setContent($messageData->getBody());
-		$this->handleAttachments($userId, $messageData, $message);
+		$this->handleAttachments($account->getMailAccount()->getUserId(), $messageData, $message);
 
 		$transport = $this->smtpClientFactory->create($account);
 		// build mime body
@@ -174,7 +172,7 @@ class MailTransmission implements IMailTransmission {
 		try {
 			$mail->send($transport, false, false);
 		} catch (Horde_Mime_Exception $e) {
-			throw new ServiceException('Could not send message', 0, $e);
+			throw new ServiceException('Could not send message: ' . $e->getMessage(), $e->getCode(), $e);
 		}
 
 		$this->eventDispatcher->dispatch(
@@ -268,8 +266,13 @@ class MailTransmission implements IMailTransmission {
 			);
 
 			$message->setInReplyTo($repliedMessage->getMessageId());
-		} catch (DoesNotExistException|Horde_Imap_Client_Exception $e) {
+		} catch (DoesNotExistException $e) {
 			// Nothing to do
+		} catch (Horde_Imap_Client_Exception $e) {
+			$this->logger->logException($e, [
+				'level' => ILogger::WARN,
+				'messages' => 'Could not find draft message: ' . $e->getMessage(),
+			]);
 		}
 
 		return $message;
@@ -288,8 +291,10 @@ class MailTransmission implements IMailTransmission {
 	 * @param string $userId
 	 * @param NewMessageData $messageData
 	 * @param IMessage $message
+	 *
+	 * @return void
 	 */
-	private function handleAttachments(string $userId, NewMessageData $messageData, IMessage $message) {
+	private function handleAttachments(string $userId, NewMessageData $messageData, IMessage $message): void {
 		foreach ($messageData->getAttachments() as $attachment) {
 			if (isset($attachment['isLocal']) && $attachment['isLocal']) {
 				$this->handleLocalAttachment($userId, $attachment, $message);
@@ -352,5 +357,4 @@ class MailTransmission implements IMailTransmission {
 			$message->addAttachmentFromFiles($file);
 		}
 	}
-
 }
